/**
   A simple generic List abstract data type.

   Here is the public List interface.
      public int size()
      public boolean isEmpty()
      public T getFirst()
      public void add(T element)
      public T remove()

   Notice that the public interface to a List does not in any
   way let you step through the list. There is no notion of
   a "current" or a "next" item in the list. The only publicly
   accessible element of the list is the first item.

   So any kind of iterator for a list must somehow get
   access to the internal structure of the list. The only
   way to do this is have the Iterator class for any List
   class be an internal, nested class, inside of the List
   class implementation.
*/
import java.util.Iterator;

public abstract class List<T> implements Iterable<T>
{
   public abstract int size();

   public boolean isEmpty()
   {
      return ( 0 == size() );
   }

   public abstract T getFirst();

   public abstract void add(T element);

   public abstract T remove();


   public String toString()
   {
      String result = "[ ";
      // use the iterator
      for (T e : this)
      {
         result += e + " ";
      }
      result += "]";
      return result;
   }


   // Provide an abstract "implementation" of the Iterable interface.
   public abstract Iterator<T> iterator();

}//List
