/**
   This file combines the ideas of Comparable
   and Iterable.

   This file contains a very generic version of
   the max() method.

   This version of max can find the maximum element
   in any iterable container of comparable elements.
*/
import java.util.Iterator;
import java.util.Comparator;

public class Max
{
   /**
      Find the largest element of any (nonempty)
      iterable container of comparable elements.
   */
   public static <T extends Comparable<T>> T max(Iterable<T> container)
   {
      Iterator<T> it = container.iterator();
      // Assume that the container is not empty.
      T largest = it.next();

      while( it.hasNext() )
      {
         T temp = it.next();
         if ( temp.compareTo( largest ) > 0 )
         {
            largest = temp;
         }
      }
      return largest;
   }



   /**
      Find the largest element of any (nonempty)
      iterable container of elements for which
      there is a comparator.
   */
   public static <T> T max(Iterable<T> container, Comparator<T> cmp)
   {
      Iterator<T> it = container.iterator();
      // Assume that the container is not empty.
      T largest = it.next();

      while( it.hasNext() )
      {
         T temp = it.next();
         if ( cmp.compare( temp, largest ) > 0 )
         {
            largest = temp;
         }
      }
      return largest;
   }

}
