/*
   This program demonstrates some of the difficulties
   with Java's generics. In this code I'm trying to use
   an interface type for the parameters to max() (why I
   would want to do that doesn't matter here).

   If the parameter type is the raw interface type Comparable,
   then you get lots of compiler warnings, and you may get a
   runtime error.

   If the parameter type is Comparable<Integer>, then the
   code won't compile! But if you tweak the declaration of
   max(), you can get the code to compile with a lot of
   warning. But you can suppress the warnings, the code
   compiles, and then runs fine! So maybe all the errors
   and warning from the compiler are wrong? But they
   are not, because of the potential for runtime errors.

   The moral of this is that many Java programs will tweak
   generic types until things compile, suppress all the
   warning, and then just hope for the best, which is not
   wise.

   There ought to be a way to use the parameterized interface
   type Comparable<Integer> for the parameters to max(), but
   I don't know a way to do that.
*/

//@SuppressWarnings("rawtypes")
public class MaxTest
{
   public static void main(String[] args)
   {
      Comparable c1 = new Integer(5);        // raw type warning for c1
      Comparable c2 = new Integer(6);        // raw type warning for c2
      System.out.println( Max.max(c1, c2) ); // unchecked method invocation

      c2 = new Double(2.0);
      System.out.println( Max.max(c1, c2) ); // unchecked method invocation

      Comparable<Integer> c3 = new Integer(7);
      Comparable<Integer> c4 = new Integer(8);
      System.out.println( Max.max(c3, c4) );   // Doesn't compile!!
                                               // unless you declare max() this way
                                               //   <T extends Comparable> T max(T a, T b)
   }
}
