/*
   This file demonstrates a simplistic kind of "generic method".
   The generic method defined below is simplistic because the
   method does not need to know anything about the type of the
   method's actual parameter. The method just does a simple
   calculation on an array's length. The method doesn't need
   to know anything about the type of the objects in the array.
*/

public class Section5_2
{
   // 1. A simple method on arrays of ints.
   public static int middle(int[] data)
   {
      if (data == null || data.length == 0)
      {
         throw new IllegalArgumentException();
      }
      else
      {  // return an element near the center of the array
         return data[data.length/2];
      }
   }


   // 2. This is an overloaded char version of the above method.
   public static char middle(char[] data)
   {
      if (data == null || data.length == 0)
      {
         throw new IllegalArgumentException();
      }
      else
      {  // return an element near the center of the array
         return data[data.length/2];
      }
   }


   // 3. This is a "generic" version of the above two methods,
   //    but it doesn't work with arays of primitives.
   //   (This is "generic" in the old (pre Java 1.5) sense of "Java Generics".)
   public static Object middle(Object[] data)
   {
      if (data == null || data.length == 0)
      {
         return null;
      }
      else
      {  // return an element near the center of the array
         return data[data.length/2];
      }
   }


   // 4. This is a true "Java Generic" version of the above methods.
   //    Notice how it declares a type variable T.
   public static <T> T middleG(T[] data)
   {
      if (data == null || data.length == 0)
      {
         return null;
      }
      else
      {  // return an element near the center of the array
         return data[data.length/2];
      }
   }



   public static void main(String[ ] args)
   {
      Integer[] intArray = {1, 2, 3, 4, 5};     // use autoboxing
      Character[] charArray = {'a', 'b', 'c'};  // use autoboxing

      // We can use our (old style) "generic" method with either array.
      // (What happens if you remove the casts?)
      int  j = (Integer)middle( intArray );
      char w = (Character)middle( charArray );

      System.out.println("j = " + j);
      System.out.println("w = " + w);


      // The cast needed on the return value from the "generic" method
      // is very undesirable. If we use the truely generic method, then
      // we don't need this cast of the return value.
      j = middleG( intArray );  // we are still using auto-unboxing
      w = middleG( charArray ); // we are still using auto-unboxing

      System.out.println("j = " + j);
      System.out.println("w = " + w);

      // The generic version works on arays of doubles or Strings also.
      // (Notice the syntax for an anonymous array literal.)
      double d = middleG( new Double[]{1.1, 2.2, 3.3, 4.4, 5.5} );
      System.out.println("d = " + d);

      String s = middleG( new String[]{"a", "ab", "abc", "abcd", "abcde"} );
      System.out.println("s = " + s);

   }//main()
}//Section5_2
