/**
   In Java, an interface is a "promise", or a "contract",
   to implement a specific set of behaviors in a class.

   More specifically, an interface is a list of (abstract)
   method declarations without definitions (method bodies).

   When a class implements an interface, the class must give
   a definition to each method declared in the interface.

   Interfaces are also types, so when a class A implements
   an interface B, then instances of A have both the type A
   and the type B (an A object "is a" B).

   When we use Interfaces as types, they classify objects
   by what they can "do" (as opposed to classes, which
   classify objects by what they "are").


          Object
             |
             |
             A      C
            / \    /|
           /   \  / |
          B     D   F
         /    / |  / \
        /    /  | /  /
        \   E   H   /
         \         /
          \       /
           \     /
            \   /
             \ /
              G
*/
class A
{
   public void methodA() {System.out.println("A");}
}

class B extends A
{
   public void methodB() {System.out.println("B");}
}

interface C
{
   public void methodC1();
   public void methodC2();
}

class D extends A implements C
{
   public void methodC1() {System.out.println("C1 from D");}
   public void methodC2() {System.out.println("C2 from D");}
   public void methodD()  {System.out.println("D");}
}

class E extends D
{
   public void methodE() {System.out.println("E");}
}

interface F extends C
{
   public void methodF();
}

class G extends B implements F
{
   public void methodC1() {System.out.println("C1 from G");}
   public void methodC2() {System.out.println("C2 from G");}
   public void methodF()  {System.out.println("F from G");}
   public void methodG()  {System.out.println("G");}
}

class H extends D implements F
{
   public void methodA()  {System.out.println("A from H");}
   //  inherit methodC1 from class D
   public void methodC2() {System.out.println("C2 from H");}
   public void methodF()  {System.out.println("F from H");}
   public void methodH()  {System.out.println("H");}
}


public class What_Are_Interfaces
{
   public static void main(String[] args)
   {
      H h = new H();
      F f = h;
      D d = h;
      C c = h;
      A a = h;
      // What methods can you call on each of a, c, d, f, and h?
      h.methodC2();
      f.methodC2();
      d.methodC2();  // this one is "polymorphic"
      c.methodC2();  // this one is "polymorphic"
//    a.methodC2();
      ((C)a).methodC2();  // this is unusual in several ways


      E e = new E();
        f = (F)e;    // Notice that there is no way that e can refer to
                     // an object that is an F, but the cast is allowed.
//      h = (H)e;    // But this cast is not allowed.
   }
}
