/*
    This program use a parameterized generic class as the return type for
    the methods methodA(), methodB() and methodC(). There is only one
    definition of the parameterized class Box<T1, T2, T3>, but this definition
    can be "instantiated" as three (or more) different kinds of Box classes.

    Notice how the definition of the Box class uses abstract "type variables",
    but in the declaration of the method return types, the type variables are
    replaced by concrete class (i.e., type) names. This is analogous to (but not
    the same as) the distinction between formal parameters in a method definition
    and the actual parameters in a method call.

    This technique of parameterized classes has the advantage of the previous
    example in that it eliminates the redundant code that was in the first example.
    But this technique has the advantage of the first example that each method clearly
    has a distinctly different return type.
*/

public class MotivateGenerics3
{
   public static Box<Integer, Integer, String> methodA()  // returns two Integers and a String
   {
      return new Box<Integer, Integer, String>( 101, 2002, "a test" ); // uses autoboxing
   }

   public static Box<Integer, Double, String> methodB()  // returns an Integer, a Double, and a String
   {
      return new Box<Integer, Double, String>( 101, 2.02, "another test" ); // uses autoboxing
   }

   public static Box<Boolean, Boolean, String> methodC()  // returns two Booleans and a String
   {
      return new Box<Boolean, Boolean, String>( false, true, "more testing" ); // uses autoboxing
   }

   static class Box<T1, T2, T3>  // parameterized generic inner class,
   {                               // used for the return type of methodA(), methodB(), methodC()
      private T1 x;
      private T2 y;
      private T3 z;

      public Box(T1 x, T2 y, T3 z)
      {
         this.x = x;
         this.y = y;
         this.z = z;
      }

      public T1 getX() { return x; }
      public T2 getY() { return y; }
      public T3 getZ() { return z; }
   }//inner class Box<T1, T2, T3>


   public static void main(String[] args)
   {
      Box<Integer, Integer, String> box1 = methodA(); // return values stored in a parameterized Box object
      int i = box1.getX().intValue();
      System.out.println( "the first integer returned by methodA is " + i );

      Box<Boolean, Boolean, String> box2 = methodC(); // return values stored in a parameterized Box object
      boolean b = box2.getX().booleanValue();
      System.out.println( "the first boolean returned by methodC is " +  b );
   }
}//MotivateGenerics3
