import java.util.Iterator;
import java.util.NoSuchElementException;

/**
   This iterable iterator represents the "range" of
   numbers between start and end in increments of inc.

   Use it in a "for-each" loop like this
      for (int i : new Range(0, 20))
      {

		}
*/
public class Range implements Iterator<Integer>, Iterable<Integer>
{
  private int start;
  private int end;
  private int inc;

  public Range(Integer x, Integer y)
  {
    start = x;
    end = y;
    inc = 1;  // default increment is 1
  }

  public Range(Integer x, Integer y, Integer i)
  {
    start = x;
    end = y;
    inc = i;
  }

  public boolean hasNext()
  {
    return start <= end;
  }

  public Integer next()
  {
    if ( ! this.hasNext() )
       throw new NoSuchElementException();

    int result = start;
    start += inc;
    return result;
  }

  public void remove()
  {
    throw new UnsupportedOperationException ();
  }

  public Iterator<Integer> iterator()
  {
    return this;
  }
}
